@@ -274,13 +274,8 @@ php_stream *php_stream_zip_open(struct zip *arch, struct zip_stat *sb, const cha
 
 /* {{{ php_stream_zip_opener */
 php_stream *php_stream_zip_opener(php_stream_wrapper *wrapper,
-#if PHP_VERSION_ID < 50600
-											char *path,
-											char *mode,
-#else
 											const char *path,
 											const char *mode,
-#endif
 											int options,
 											char **opened_path,
 											php_stream_context *context STREAMS_DC TSRMLS_DC)
