@@ -199,6 +199,35 @@ static int php_zip_ops_stat(php_stream *stream, php_stream_statbuf *ssb) /* {{{
 }
 /* }}} */
 
+#if LIBZIP_ATLEAST(1,9,1)
+/* {{{ php_zip_ops_seek */
+static int php_zip_ops_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffset)
+{
+	int ret = -1;
+	STREAM_DATA_FROM_STREAM();
+
+	if (self->zf) {
+		ret = zip_fseek(self->zf, offset, whence);
+		*newoffset = zip_ftell(self->zf);
+	}
+
+	return ret;
+}
+/* }}} */
+
+/* with seek command */
+php_stream_ops php_stream_zipio_seek_ops = {
+	php_zip_ops_write, php_zip_ops_read,
+	php_zip_ops_close, php_zip_ops_flush,
+	"zip",
+	php_zip_ops_seek, /* seek */
+	NULL, /* cast */
+	php_zip_ops_stat, /* stat */
+	NULL  /* set_option */
+};
+#endif
+
+/* without seek command */
 php_stream_ops php_stream_zipio_ops = {
 	php_zip_ops_write, php_zip_ops_read,
 	php_zip_ops_close, php_zip_ops_flush,
@@ -210,7 +239,7 @@ php_stream_ops php_stream_zipio_ops = {
 };
 
 /* {{{ php_stream_zip_open */
-php_stream *php_stream_zip_open(struct zip *arch, const char *path, const char *mode STREAMS_DC)
+php_stream *php_stream_zip_open(struct zip *arch, struct zip_stat *sb, const char *mode, zip_flags_t flags STREAMS_DC)
 {
 	struct zip_file *zf = NULL;
 
@@ -222,7 +251,7 @@ php_stream *php_stream_zip_open(struct zip *arch, const char *path, const char *
 	}
 
 	if (arch) {
-		zf = zip_fopen(arch, path, 0);
+		zf = zip_fopen_index(arch, sb->index, flags);
 		if (zf) {
 			self = emalloc(sizeof(*self));
 
@@ -230,8 +259,15 @@ php_stream *php_stream_zip_open(struct zip *arch, const char *path, const char *
 			self->zf = zf;
 			self->stream = NULL;
 			self->cursor = 0;
-			stream = php_stream_alloc(&php_stream_zipio_ops, self, NULL, mode);
-			stream->orig_path = estrdup(path);
+#if LIBZIP_ATLEAST(1,9,1)
+			if (zip_file_is_seekable(zf) > 0) {
+				stream = php_stream_alloc(&php_stream_zipio_seek_ops, self, NULL, mode);
+			} else
+#endif
+			{
+				stream = php_stream_alloc(&php_stream_zipio_ops, self, NULL, mode);
+			}
+			stream->orig_path = estrdup(sb->name);
 		}
 	}
 
@@ -314,7 +350,14 @@ php_stream *php_stream_zip_opener(php_stream_wrapper *wrapper,
 			self->zf = zf;
 			self->stream = NULL;
 			self->cursor = 0;
-			stream = php_stream_alloc(&php_stream_zipio_ops, self, NULL, mode);
+#if LIBZIP_ATLEAST(1,9,1)
+			if (zip_file_is_seekable(zf) > 0) {
+				stream = php_stream_alloc(&php_stream_zipio_seek_ops, self, NULL, mode);
+			} else
+#endif
+			{
+				stream = php_stream_alloc(&php_stream_zipio_ops, self, NULL, mode);
+			}
 
 			if (opened_path) {
 				*opened_path = zend_string_init(path, strlen(path), 0);
